class Player extends GameObj {

  boolean touchingGround = true;
  PVector lastFrame;
  ArrayList[] inventory;
  //ArrayList<Terrain> inventory;

  Player() {
    super(new PVector(0, -100), new PVector(30, 30));
    inventory = new ArrayList[36];
    for(int i = 0; i < 36; i++){
      inventory[i] = new ArrayList();
    }
    acceleration.x = 0;
    acceleration.y = 0.3; // gravity
    this.S = 0.8;
    this.H = 0.73;
  }

  void update() {
    lastFrame = new PVector(position.x, position.y);

    if (touchingGround) {
      if (rightKey)velocity.x += .6;
      if (leftKey)velocity.x += -.6;
    } 
    else {
      if (rightKey)velocity.x += .06;
      if (leftKey)velocity.x += -.06;
    }

    velocity.x += acceleration.x;
    velocity.y += acceleration.y;

    if (touchingGround) velocity.x /= 1.1;

    position.x += velocity.x;
    position.y += velocity.y;

    checkTerrainCollision();

    if (position.y + (dimensions.y/2) >= 0) { // Replace with terrain check
      position.y = -dimensions.y/2; 
      setRest();
    }
    // bounds checking
    if(position.x < -6200) position.x = -6200;
    if(position.x > 6200) position.x = 6200;
    if(position.y < -1500) position.y = -1500;
  }


  void jump() {
    if (touchingGround) {
      //println("JUMPING!");
      velocity.y -= 9;
      touchingGround = false;
    }
  }

  void setRest() {
    touchingGround = true;
    velocity.y = 0;
  }
  
  void addToInventory(Terrain t){
    int index = (int)(t.H * 36);
    //println("adding terrain with H " + t.H + " to player's inventory at index " + index);
    inventory[index].add(t);
  }

  ArrayList<Terrain> terrainToCheck = new ArrayList<Terrain>();
  void checkTerrainCollision() {
    boolean collided = false;
    terrainToCheck = new ArrayList<Terrain>();
    for (float i = this.position.x + (terrainSize*2); i >= this.position.x - (terrainSize*3); i -= terrainSize) {
      //println("Player position: " + this.position);
      for (float j = this.position.y + (terrainSize*2); j >= this.position.y - (terrainSize*3); j -= terrainSize) {
        terrainToCheck.add(getTerrainAt(new PVector(i, j)));
      }
    }

    for (Terrain obj : terrainToCheck) {
      if (obj != null) {
        if (grabbed != null && grabbed.id == obj.id && !grabbed.locked) {
          addToInventory(grabbed);
          //println("GRABBED Lineweight: " + grabbed.stweight);
          //println("Attempting to remove grabbed object at position " + grabbed.position);
          //printWorldIndex(grabbed.position);
          removeTerrainAt(new PVector(grabbed.position.x, grabbed.position.y));
          grabUpdateChildren(grabbed);
          grabbed = null;
        } 
        else {

          float verticalDistance = abs(this.position.y - obj.position.y);
          float horizontalDistance = abs(this.position.x - obj.position.x);
          obj.B = 1;
          if ((verticalDistance  < (this.dimensions.y/2) + (obj.dimensions.y/2) ) &&
            (horizontalDistance  < (this.dimensions.x/2) + (obj.dimensions.x/2) )) {
            // WE ARE INSIDE IT, CAPTAIN ! Where shall we go?
            //println("this position: " + this.position);
            //println(verticalDistance + ", " + horizontalDistance + " ||| " + ((this.dimensions.y/2) + (obj.dimensions.y/2)) + ", " + ((this.dimensions.x/2) + (obj.dimensions.x/2)));
            collided = true;
            //obj.S = 0.7;

            float verticalDiscrep = abs(lastFrame.y - obj.position.y) -((this.dimensions.y/2) + (obj.dimensions.y/2)); //USE LAST FRAME
            float horizontalDiscrep = abs(lastFrame.x - obj.position.x) -((this.dimensions.x/2) + (obj.dimensions.x/2));
            if (verticalDiscrep >= horizontalDiscrep) { // fall on top (or bottom)
              //println(obj.position.y + " - " + ((this.dimensions.y/2) + (obj.dimensions.y/2)) + " = " + (obj.position.y - ((this.dimensions.y/2) + (obj.dimensions.y/2))));
              //println("if... " + (this.position.y - (this.dimensions.y/2)) + " <? " + (obj.position.y - (obj.dimensions.y/2)));
              if ((this.position.y - (this.dimensions.y/2)) > (obj.position.y - (obj.dimensions.y/2))) {
                // Bump head? I dunno
                this.position.y = obj.position.y + ((this.dimensions.y/2) + (obj.dimensions.y/2));
                this.velocity.y = 0;
              } 
              else {
                this.position.y = obj.position.y - ((this.dimensions.y/2) + (obj.dimensions.y/2));
                this.setRest();
              }
            } 
            else if (verticalDiscrep != 0) { // collide into wall from the side if we are not already on top of it
              if (this.position.x < obj.position.x) { // to the left
                //println(verticalDiscrep + " >=? " + horizontalDiscrep);
                //println(this.position.x + " <? " + obj.position.x);
                this.position.x = obj.position.x - ((this.dimensions.x/2) + (obj.dimensions.x/2));
                //println("11111 Collided horizontally, setting xVelocity to 0");
                this.velocity.x = 0;
              } 
              else if (this.position.x > obj.position.x) { // to the right
                this.position.x = obj.position.x + ((this.dimensions.x/2) + (obj.dimensions.x/2));
                //println("22222Collided horizontally, setting xVelocity to 0");
                this.velocity.x = 0;
              }
            }
          }
        }
        if (collided) this.B = 0.9;
        else this.B = 0.7;
      }
    }
  }
}

